.RSJava.symbol <-
function(name)
{
paste("RS_JAVA_",name,sep="")
}


setClass("JavaConfiguration",
          representation(classPath = "character",
                         libraryPath = "character",
                         properties = "character",
                         options = "character",
                         args = "character"))


defaultJavaPath =
function(varName = "CLASSPATH") {
        # note that this requires R >= 1.2.0
        # with the Sys.getenv. We could just use getenv
        # but that is being deprecated so we put in both
        # as a reminder.
    classPath <- 
        if(exists("Sys.getenv")) Sys.getenv(varName) 
        else getenv(varName) 
    sep <- ifelse(.Platform$OS.type != "windows", ":", ";")
    classPath <- strsplit(classPath, sep)[[1]]
    classPath[nzchar(classPath) & !duplicated(classPath)]
}

JavaConfiguration =
  function(classPath = defaultJavaPath(),
           libraryPath = defaultJavaPath("LD_LIBRARY_PATH"),
           properties = character(0), options = character(0),
           args = character(0))
{
    new("JavaConfiguration", classPath=classPath,
        libraryPath=libraryPath, properties=properties,
        options=options, args=args)
}

setMethod("merge", c("JavaConfiguration", "JavaConfiguration"),
          function(x, y, ...) {
# This combines the elements of both JavaConfiguration objects into
# a single JavaConfiguration object. It returns an object of the
# same class as `x'.            

            pathSeparator <- ifelse(version$os == "Win32", ";", ":") 
            
            obj = x
            obj@classPath = mergePath(x@classPath, y@classPath)
            obj@libraryPath = mergePath(x@libraryPath, y@libraryPath)            
            obj@properties = mergeProperties(x@properties, y@properties, FALSE)
            obj@options <- as.character(unique(c(x@options, y@options)))
            obj@args <- as.character(unique(c(x@args, y@args)))            
            
            obj
          })


setGeneric("collapse", function(x, ...) standardGeneric("collapse"))

setMethod("collapse", "JavaConfiguration",
          function(x, collapse = ifelse(.Platform$OS.type == "windows", ";", ":") ) {
# This is called to reduce the elements of the JavaConfiguration into strings
# that are used directly in the initialization of the JVM.
# This collapses the class and library path elements into a single string
# and puts the properties into a -Dname=value arguments.            
            
           x@classPath = paste(x@classPath, collapse=collapse)
           x@libraryPath = paste(x@libraryPath, collapse=collapse)
               # Convert these into -Dname=value while putting the java.library.path valueinto them.
           x@properties <- mergeProperties(c("java.library.path" = x@libraryPath), x@properties, convert = TRUE)
           x@properties <- c(x@properties, x@options)

           x
})


.JavaInit <-
  ## initialize the S-Java interface and start the Java Virtual Machine.
  ##
  ## Argument `classPath', if supplied, is a vector of directory paths
  ## to be added to the class path for the Java Virtual Machine when
  ## it starts up.  (This path always includes the locations of the
  ## classes used by the interfaceManager and the standard Omega classes.)
  ##
  ## This function is called from the .Java interface; except for the
  ## first call, it normally detects the `.JavaInitialized' object in
  ## frame 0 and does nothing.  On the first call, it collects the
  ## Java configuration information by a call to `.JavaConfig', uses
  ## that to set a shell variable and for arguments to the C routine
  ## that starts the VM.
  ##
  ## See `.JavaConfig' for ways to customize the configuration
  ## parameters of the Virtual Machine.
  function(..., config = NULL, default = javaConfig(), verbose = FALSE, callbackHandler = javaHandlerGenerator())
{
    if(exists(".JavaInitialized", frame=0))
        return(TRUE)

    settings = JavaConfiguration(...)

    if(is.null(config))
      config = settings
    else
      config = merge(settings, config)    

#XXX apparently needed here to force the method dispatching to work correctly!!!
class(default)
    config = merge(config, default)

    configuration = config
    
    config = collapse(config)

    if(verbose) 
      print(config)
    
      # start the virtual machine and the Omegahat interpreter, initializing different
      # classes and internal variables.
    .C("s_start_VM", config@classPath, config@properties,
       length(config@properties), config@options,
       length(config@options), PACKAGE="SJava")
      # establish an evaluator function that brokers the calls to S functions.
     if(!is.null(callbackHandler))
         .Call("RJava_setDefaultHandlerFunction", callbackHandler,
               PACKAGE="SJava")

     setDefaultConverters()
    
     assign(".JavaInitialized" , TRUE, immediate=TRUE)

     invisible(configuration)
}

.JavaAttach <-
#
# Intended to be called when running R from within Java.
# This does the initialization that 
function()
{
 val <- .Call("s_register_VM", PACKAGE="SJava")
 if(val)
   assign(".JavaInitialized", TRUE, immediate = TRUE)
 else {
     stop("Couldn't attach to running JVM")
 }

return(val)
}


.JavaTerminate <-
function()
{
 if(isJavaInitialized())
   .C("RS_JAVA_terminateJava", PACKAGE="SJava")

 if(exists(".JavaInitialized"))
   remove(list = ".JavaInitialized")
 
 TRUE
}
