/* #line 505 "ConverterExamples.cweb" */
#include "Converters.h"
#include "ConverterExamples.h"
#include "RSCommon.h"

#include "RtoJava.h"
#include "JNIUtils.h"
#include "CtoJava.h" /* MUST_GET_METHOD_ID  */

#include "RReferenceCall.h"

#include <string.h>

#include "ManagerInterfaceUtils.h"

/* #line 137 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(callRConverterFunction)(USER_OBJECT_ fun, jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter, ConverterOpType opType);

/* #line 235 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(RfunctionConverter)(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter);

/* #line 271 "ConverterExamples.cweb" */
jboolean
RS_JAVA(toJavaFunctionConverterMatch)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env,  RSToJavaConverter *converterEl);

/* #line 295 "ConverterExamples.cweb" */
jobject
RS_JAVA(toJavaFunctionConverter)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env,  RSToJavaConverter *converterEl);
/* #line 319 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(callToJavaConverterFunction)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env,  RSToJavaConverter *converterEl, ConverterOpType opType);
/* #line 353 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(createToJavaFunctionConverterArgs)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env);

/* #line 417 "ConverterExamples.cweb" */
USER_OBJECT_
primitiveTwoLevelArrayConverter(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter);
jboolean
primitiveTwoLevelArrayMatch(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter);

/* #line 77 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(RealVariableConverter)(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter)
{
    jmethodID ValuesID;
    jobject array;
    USER_OBJECT_ ans;
    ValuesID = VMENV GetMethodID(env, type, "getValues", "()[D");

    array = VMENV CallObjectMethod(env, obj, ValuesID);
    ans = s_from_java_basic(env, array, "[D", TRUE, FALSE);

    return ans;
}



/* #line 108 "ConverterExamples.cweb" */
jboolean
RS_JAVA(functionConverterMatch)(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter)
{
    USER_OBJECT_ ans;
    USER_OBJECT_ fun;
    USER_OBJECT_ functions;
    int ok;


    /* Get the matching function from the user data. */
    /* Have to handle a closure that does both conversion and matching! */
    functions = (USER_OBJECT_)  (converter->userData);
    fun = VECTOR_ELT(functions, MatchFunction);

    ans = RS_JAVA(callRConverterFunction)(fun, obj, type, env, converter, MatchFunction);
    ok = LOGICAL_DATA(ans)[0];

    return ok ? JNI_TRUE : JNI_FALSE;
}

/* #line 202 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(setFunctionConverter)(USER_OBJECT_ funs,
                              USER_OBJECT_ description,
                              USER_OBJECT_ fromJava,
                              USER_OBJECT_ autoArray,
                              USER_OBJECT_ position)
{
    int index;
    USER_OBJECT_ ans;
    const char *desc = NULL;
    R_PreserveObject(funs);

    index = INTEGER_DATA(position)[0] - 1;

    if(GET_LENGTH(description))
        desc = CHAR_DEREF(STRING_ELT(description, 0));
    if(desc && desc[0])
        desc = strdup(desc);

    if(LOGICAL_DATA(fromJava)[0]) {
        addFromJavaConverterInfo(RS_JAVA(functionConverterMatch),
                                 RS_JAVA(RfunctionConverter),
                                 LOGICAL_DATA(autoArray)[0],
                                 funs, desc, &index);
    } else {
        addToJavaConverterInfo(RS_JAVA(toJavaFunctionConverterMatch),
                               RS_JAVA(toJavaFunctionConverter), JNI_TRUE,
                               funs, desc, &index);
    }

    PROTECT(ans = NEW_INTEGER(1));
    INTEGER_DATA(ans)[0] = index;
    UNPROTECT(1);
    return ans;
}

/* #line 248 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(RfunctionConverter)(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter)
{
    USER_OBJECT_ fun;
    USER_OBJECT_ functions;
    USER_OBJECT_ ans;

    functions = (USER_OBJECT_)  (converter->userData);
    fun = VECTOR_ELT(functions, ConverterFunction);
    ans = RS_JAVA(callRConverterFunction)(fun, obj, type, env, converter, ConverterFunction);

    return ans;
}

/* #line 151 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(callRConverterFunction)(USER_OBJECT_ fun, jobject obj,
				jclass type, JNIEnv *env,
				RSFromJavaConverter *converter,
				ConverterOpType opType)
{
    USER_OBJECT_ args;
    USER_OBJECT_ rclassName;
    USER_OBJECT_ call;
    USER_OBJECT_ ans;
    int errorOccurred;

    jstring tmp;
    const char *klassName;
    /* Now create the R arguments to the matching function. */
    PROTECT(args = NEW_LIST(2));
    SET_VECTOR_ELT(args, 0, anonymousAssign(env, obj, type));
    PROTECT(rclassName = NEW_CHARACTER(1));

    tmp = get_object_class_name(env, obj);
    klassName = VMENV GetStringUTFChars(env, tmp, NULL);
    SET_STRING_ELT(rclassName, 0, COPY_TO_USER_STRING(klassName));
    VMENV ReleaseStringUTFChars(env, tmp, klassName);

    SET_VECTOR_ELT(args, 1, rclassName);

    PROTECT(call = RJava(createCall)(fun, args));

#if defined(_R_)
    ans = R_tryEval(call, R_GlobalEnv, &errorOccurred);
    if(errorOccurred) {
        ans = NULL_USER_OBJECT;
    }
#else

#endif

    UNPROTECT(3);

    return ans;
}


/* #line 424 "ConverterExamples.cweb" */
static jclass PrimitiveJavaArrayClasses[8];

jboolean
primitiveTwoLevelArrayMatch(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter)
{
    int i;
    //XXX remember for close down.
    if(PrimitiveJavaArrayClasses[0] == NULL || 0)  {
        char buf[4]="[[I\0";
        char *typeSigs[] = {"Z", "B", "C", "S", "I", "J", "F", "D"};
        for(i = 0; i < 8; i++) {
            jclass tmp;
            buf[2] = typeSigs[i][0];
            tmp = VMENV FindClass(env, buf);
            PrimitiveJavaArrayClasses[i] = VMENV NewGlobalRef(env, tmp);
        }
    }

    for(i = 0; i < 8 ; i++) {
        if(VMENV IsSameObject(env, PrimitiveJavaArrayClasses[i], type) == JNI_TRUE) {
#ifdef LDEBUG
            REprintf("[primitiveTwoLevelArrayMatch] match for %d (%p = %p)\n", i, PrimitiveJavaArrayClasses[i], type);
#endif
            return JNI_TRUE;
        }
    }

    return JNI_FALSE;
}

USER_OBJECT_
primitiveTwoLevelArrayConverter(jobject obj, jclass type, JNIEnv *env, RSFromJavaConverter *converter)
{
    int numEls, i;
    USER_OBJECT_ ans, val;
    jobject el;

    numEls = VMENV GetArrayLength(env, obj);

    PROTECT(ans = NEW_LIST(numEls));

    for(i = 0; i < numEls; i++) {
        el = VMENV GetObjectArrayElement(env, obj, i);
        val = s_from_java_basic(env, el, NULL, JNI_TRUE, JNI_FALSE);
        SET_VECTOR_ELT(ans, i, val);
    }

    UNPROTECT(1);
    return ans;
}

jboolean
RS_JAVA(toJavaFunctionConverterMatch)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env,  RSToJavaConverter *converter)
{
    USER_OBJECT_ ans;
    int ok;

    /* Get the matching function from the user data. */
    /* Have to handle a closure that does both conversion and matching! */

    ans = RS_JAVA(callToJavaConverterFunction)(obj, type, jtype, signature, isArray, arrayLen, env, converter, MatchFunction);
    ok = LOGICAL_DATA(ans)[0];

    return ok ? JNI_TRUE : JNI_FALSE;
}

/* #line 299 "ConverterExamples.cweb" */
jobject
RS_JAVA(toJavaFunctionConverter)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env,  RSToJavaConverter *converter)
{
    USER_OBJECT_ fun;
    USER_OBJECT_ functions;
    USER_OBJECT_ ans;
    jobject value;

    functions = (USER_OBJECT_)  (converter->userData);
    fun = VECTOR_ELT(functions, ConverterFunction);
    ans = RS_JAVA(callToJavaConverterFunction)(obj, type, jtype, signature,
											   isArray, arrayLen, env,
											   converter, ConverterFunction);

	/* unhandled types, not just vectors, end up here */
    if(TYPEOF(ans) != VECSXP) {
	  value = NULL_JAVA_OBJECT;
	} else {
	  const char *val = CHAR_DEREF(STRING_ELT(VECTOR_ELT(ans, 0), 0));
	  value = getOmegahatReferenceValue(val, JNI_TRUE, env);
	}
    return value;
}

/* #line 323 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(callToJavaConverterFunction)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env,  RSToJavaConverter *converterEl, ConverterOpType opType)
{
    USER_OBJECT_ call, ans;
    USER_OBJECT_ args, fun, funs;
    int errorOccurred;

    args = RS_JAVA(createToJavaFunctionConverterArgs)(obj, type, jtype, signature, isArray, arrayLen, env);
    PROTECT(args);
    /* Now create the R arguments to the matching function. */
    funs = (USER_OBJECT_) converterEl->userData;
    fun = VECTOR_ELT(funs, opType);

    PROTECT(call = RJava(createCall)(fun, args));

#if defined(_R_)
    ans = R_tryEval(call, R_GlobalEnv, &errorOccurred);
    if(errorOccurred)
        ans = NULL_USER_OBJECT;
#else

#endif

    UNPROTECT(2);

    return ans;
}

/* #line 358 "ConverterExamples.cweb" */
USER_OBJECT_
RS_JAVA(createToJavaFunctionConverterArgs)(USER_OBJECT_ obj, jclass type, java_type jtype, const char *signature, jboolean isArray, int arrayLen, JNIEnv *env)
{
    USER_OBJECT_ args;
    USER_OBJECT_ rclassName;
    USER_OBJECT_ tmp;
    jstring tmps;
    const char *klassName;
#ifdef LDEBUG
    REprintf("creating arguments for JavaFunctionConverter args.\n");
#endif
    PROTECT(args = NEW_LIST(6));
    SET_VECTOR_ELT(args, 0, obj);

    SET_VECTOR_ELT(args, 2, tmp = NEW_LOGICAL(1));
    LOGICAL_DATA(tmp)[0] = isArray;

    SET_VECTOR_ELT(args, 3, tmp = NEW_INTEGER(1));
    INTEGER_DATA(tmp)[0] = arrayLen;

    SET_VECTOR_ELT(args, 4, tmp = NEW_INTEGER(1));
    INTEGER_DATA(tmp)[0] = jtype;

#ifdef LDEBUG
    REprintf("finished arg 4\n");
#endif

    SET_VECTOR_ELT(args, 5, tmp = NEW_CHARACTER(1));
    if(signature != NULL)
        SET_STRING_ELT(tmp, 0, COPY_TO_USER_STRING(signature));

    PROTECT(rclassName = NEW_CHARACTER(1));

#ifdef LDEBUG
    REprintf("creating class argument\n");
#endif
    if(0 && type != NULL_JAVA_OBJECT) {
        tmps = get_java_class_name(env, type);
        klassName = VMENV GetStringUTFChars(env, tmps, NULL);
        SET_STRING_ELT(rclassName, 0, COPY_TO_USER_STRING(klassName));
	VMENV ReleaseStringUTFChars(env, tmps, klassName);
    }
    SET_VECTOR_ELT(args, 1, rclassName);

    UNPROTECT(2);
#ifdef LDEBUG
    REprintf("finished creating arguments\n");
#endif

    return args;
}




