/* #line 7 "RReferenceCall.cweb" */
#include "RReferenceCall.h"
#include "RtoJava.h"
#include "RStoJava.h"
#include "RForeignReference.h"

#define VMENV (*env)->

/* #line 60 "RReferenceCall.cweb" */
enum {IDENTIFIER=0, METHOD_NAME, RETURN_TYPE, SIGNATURE, THIS, ARGUMENTS, NUM_SLOTS};

/* #line 182 "RReferenceCall.cweb" */
static SEXP defaultMethodHandler = NULL;


/* #line 83 "RReferenceCall.cweb" */
#ifdef REGISTER_NATIVES
static
#endif
jobject JNICALL Java_org_omegahat_R_Java_RForeignReference_reval
(JNIEnv *env, jobject rref, jstring jid, jobjectArray args, jstring jmethodName, jstring jreturnClass, jobjectArray  jsignature)
{
    SEXP details, userAns;
    int numArgs = 0;
    const char *sig;
    jobject ans = NULL_JAVA_OBJECT;

    RS_lockEvaluator(env, JNI_TRUE, JNI_TRUE);

    PROTECT(details = NEW_LIST((int) NUM_SLOTS));

    RS_JAVA(addStringValue)(env, jid,          details, IDENTIFIER);
    RS_JAVA(addStringValue)(env, jmethodName,  details, METHOD_NAME);
    RS_JAVA(addStringValue)(env, jreturnClass, details, RETURN_TYPE);
    RS_JAVA(addStringValues)(env, jsignature,   details, SIGNATURE);


    /* Convert the arguments. */
    if(args != NULL_JAVA_OBJECT)
        numArgs = VMENV GetArrayLength(env, args);

    if(numArgs > 0) {
        USER_OBJECT_ remoteArgs;
        jobject el;
        int i;
        remoteArgs = NEW_LIST(numArgs) ;
        SET_VECTOR_ELT(details, ARGUMENTS, remoteArgs);
        for(i = 0; i < numArgs ; i++) {
            el = VMENV GetObjectArrayElement(env, args, i);
            SET_VECTOR_ELT(VECTOR_ELT(details, ARGUMENTS), i, s_from_java_basic(env, el, NULL, TRUE, TRUE));
        }
    }

    //XXX must release this object. Really want to use external pointers at this point.
    SET_VECTOR_ELT(details, THIS, anonymousAssign(env, rref, NULL));

    /* Convert the `this' object to a reference. */

    PROTECT(userAns = RJava(invokeRReferenceMethod)(details));
    if(GET_LENGTH(userAns) == 2) {
        /* An error. */
        const char *msg = strdup(CHAR_DEREF(STRING_ELT(VECTOR_ELT(userAns, 1), 0)));
        UNPROTECT(2); /* details */


        throw_Java_exception(msg, NULL,  env);
    } else {
        /* Convert result. */
        if(GET_LENGTH(VECTOR_ELT(userAns, 0)) > 0) {
            ans = s_to_java_basic(env, VECTOR_ELT(userAns, 0), NULL, &sig);
        }
    }

    UNPROTECT(2); /* details, userAns */
    RS_lockEvaluator(env, JNI_FALSE, JNI_TRUE);
    return ans;
}

/* #line 150 "RReferenceCall.cweb" */
#ifdef REGISTER_NATIVES
int
RSJava_registerNatives(JNIEnv *env)
{
    jclass klass;
    JNINativeMethod nm;
    int status;

    klass = VMENV FindClass(env, "org/omegahat/R/Java/RForeignReference");
    if(VMENV ExceptionCheck(env)) {
        REprintf("Can't find RForeignReference class\n");
        VMENV ExceptionDescribe(env);
    }

    nm.name = "reval";
    nm.fnPtr = Java_org_omegahat_R_Java_RForeignReference_reval;
    nm.signature= "(Ljava/lang/String;[Ljava/lang/Object;Ljava/lang/String;Ljava/lang/String;[Ljava/lang/String;)Ljava/lang/Object;";

    status = VMENV RegisterNatives(env, klass, &nm, 1);

    REprintf("Result of RegisterNatives %d\n", status);
    if(VMENV ExceptionCheck(env)) {
        VMENV ExceptionDescribe(env);
    }

    return status;
}
#endif


/* #line 288 "RReferenceCall.cweb" */
int
RS_JAVA(addStringValue)(JNIEnv *env,  jstring str, USER_OBJECT_ details, int which)
{
    const char *tmp;
    SET_VECTOR_ELT(details, which, NEW_CHARACTER(1));
    if(str == NULL_JAVA_OBJECT) {
        return which;
    }

    tmp = VMENV GetStringUTFChars(env, str, NULL);
    SET_STRING_ELT(VECTOR_ELT(details, which), 0, mkChar(tmp));
    VMENV ReleaseStringUTFChars(env, str, tmp);

    return which;
}

/* #line 316 "RReferenceCall.cweb" */
int
RS_JAVA(addStringValues)(JNIEnv *env, jobjectArray strArray, USER_OBJECT_ details, int slot)
{
    int n, i;
    const char *tmp;
    jobject el;

    n = VMENV GetArrayLength(env, strArray);

    SET_VECTOR_ELT(details, slot, NEW_CHARACTER(n));
    for(i = 0; i < n ; i++) {
        el = VMENV GetObjectArrayElement(env, strArray, i);
        tmp = VMENV GetStringUTFChars(env, el, NULL);
        SET_STRING_ELT(VECTOR_ELT(details, slot), i, COPY_TO_USER_STRING(tmp));
	VMENV ReleaseStringUTFChars(env, el, tmp);
    }

    return slot;
}


/* #line 200 "RReferenceCall.cweb" */
USER_OBJECT_
RJava(defaultHandlerFunction)()
{
    if(defaultMethodHandler == NULL) {
        PROBLEM "the Java callback handler has not been registered!"
            ERROR;
    }
    return defaultMethodHandler;
}

/* #line 212 "RReferenceCall.cweb" */
USER_OBJECT_
RJava(setDefaultHandlerFunction)(USER_OBJECT_ handler)
{
    USER_OBJECT_ old = defaultMethodHandler;

    if(old && old != NULL_USER_OBJECT) {
        R_ReleaseObject(old);
    } else
        old = NULL_USER_OBJECT;

    R_PreserveObject(handler);
    defaultMethodHandler = handler;

    return old;
}

/* #line 235 "RReferenceCall.cweb" */
USER_OBJECT_
RJava(invokeRReferenceMethod)(USER_OBJECT_ opArgs)
{
    /*XXX Need to do this for the moment to avoid the
      Error: C stack usage is too close to the limit
      message. This is related to threads, being in a different
      thread.  simpleInterface.R in tests/ illustrates this.
    */
    extern int R_CStackLimit;
    int prevStackLimit = R_CStackLimit;

    SEXP fun;
    SEXP call;
    SEXP ans;
    int errorOccurred;

    R_CStackLimit = -1;
    fun = RJava(defaultHandlerFunction)();

    /* Check whether we have a function. If not, assume it is a list and
       that the first element is a function.
    */
    if(isFunction(fun) == FALSE) {
        if(IS_VECTOR(fun)) {
            fun = VECTOR_ELT(fun, 0);
        }
    }

    call = RJava(createCall)(fun, opArgs);

    PROTECT(call);
    ans = R_tryEval(call, R_GlobalEnv, &errorOccurred);
    R_CStackLimit = prevStackLimit;
    if(errorOccurred)
        ans = NULL_USER_OBJECT;

    UNPROTECT(1);
    return ans;
}


/* #line 346 "RReferenceCall.cweb" */
SEXP
RJava(createCall)(SEXP fun, SEXP opArgs)
{
    SEXP c, call;
    int i;
    long n;

    /* Generate the appropriate R object for invoking the function with these arguments. */
    n = GET_LENGTH(opArgs);
    if(n  > 0) {
        PROTECT(c = call = allocList(n));
        for (i = 0; i < n; i++) {
            SETCAR(c, VECTOR_ELT(opArgs, i));
            c = CDR(c);
        }

        call = LCONS(fun, call);
        UNPROTECT(1);
    } else  {
        call = allocVector(LANGSXP,1);
        SETCAR(call,  fun);
    }

    return call;
}


/* #line 405 "RReferenceCall.cweb" */
USER_OBJECT_
getRReference(jobject ref, JNIEnv *env)
{
    const char *tmp;
    USER_OBJECT_ name, e, ans;
    int errorOccurred;

    USER_OBJECT_ f = RJava_defaultHandlerFunction();
    USER_OBJECT_ getReferenceFun;

    /* Get the getReference() function and set this as the function
       to call.
    */
    getReferenceFun = VECTOR_ELT(f, 4);


    /* Now call this function with the single argument, the
       reference name/identifier.
    */
    PROTECT(e = allocVector(LANGSXP, 2));
    SETCAR(e, getReferenceFun);

    /* Convert the java string identifying the reference to an R object. */
    tmp = VMENV GetStringUTFChars(env, (jstring) ref, NULL);
    PROTECT(name = NEW_CHARACTER(1));
    SET_STRING_ELT(name, 0, COPY_TO_USER_STRING(tmp));
    VMENV ReleaseStringUTFChars(env, (jstring) ref, tmp);
    SETCAR(CDR(e), name);

//XXX
    /* Now ready to call the getReference() function. */
    ans =  R_tryEval(e, R_GlobalEnv, &errorOccurred);
    if(errorOccurred)
        ans = NULL_USER_OBJECT;

    UNPROTECT(2);

    return ans;
}



/* #line 463 "RReferenceCall.cweb" */
jstring
getJavaReference(jobject reference,  JNIEnv *env)
{
    jclass klass = VMENV GetObjectClass(env, reference);
    jmethodID mid;
    jstring ans;
    mid = VMENV GetMethodID(env, klass, "getName", "()Ljava/lang/String;");

    ans = VMENV CallObjectMethod(env, reference, mid);

    return ans;
}

/* #line 378 "RReferenceCall.cweb" */
jboolean
isRReference(jobject obj, JNIEnv *env)
{
    jclass klass;
    jboolean ans;
    klass = VMENV FindClass(env, "org/omegahat/R/Java/RForeignReference");
    if(!klass) {
        PROBLEM "Can't find org.omegahat.R.Java.RForeignReference"
            ERROR;
    }
    ans = VMENV IsInstanceOf(env, obj, klass);
//XX Do we need to release this klass.

    return ans;
}





